@@ -297,6 +297,7 @@ class DataFrameSchematizer:
         self._source_units = None
         self._target_units = None
         self.uniqueness_sets = defaultdict(list)
+        self.xcalling = {}
 
     def build(self):
         """build and return schema"""
@@ -316,6 +317,14 @@ class DataFrameSchematizer:
                 "type": "array",
                 "uniqueItems": True,
             }
+
+        for colname, (ext_tabname, ext_colname) in self.xcalling.items():
+            if "xcalls" not in schema:
+                schema["xcalls"] = {}
+            schema["xcalls"][colname] = {
+                "tabname": ext_tabname,
+                "colname": ext_colname,
+            }
         return schema
 
     def _add_columns_from_json(self, jsontxt):
@@ -349,6 +358,7 @@ class DataFrameSchematizer:
         unique=False,
         mandatory=False,
         uniqueness_id=None,
+        must_exist_in=None,
         units=None,
         **kwargs,
     ):
@@ -362,6 +372,8 @@ class DataFrameSchematizer:
             types.append("null")
         if uniqueness_id:
             self.uniqueness_sets[uniqueness_id].append(name)
+        if must_exist_in:
+            self.xcalling[name] = tuple(must_exist_in.split("::"))
         # ---------------------------------------------------------
         if len(types) > 1:
             items = {"anyOf": [{"type": typ} for typ in types]}
@@ -389,7 +401,7 @@ class DataFrameSchematizer:
         3. default values are applied
         4. final validation
         """
-        _df_debug = df.copy()
+        # _df_debug = df.copy()
         schema = self.build()
         validator = jsonschema.Draft7Validator(schema)
         # =====================================================================
@@ -399,9 +411,14 @@ class DataFrameSchematizer:
         # If a data has the wrong type, this process would fail before reaching
         # validation
         # =====================================================================
+
+        # ensure line#0 is first line of data
+        offset = {True: -1, False: 0}
+        df = df.reset_index(drop=True)
+        df.index += offset[self._is_units]
         initial_without_urow = df.copy()
         if self._is_units:
-            initial_without_urow = initial_without_urow.drop(0)
+            initial_without_urow = initial_without_urow.iloc[1:]
         # check **types only** before any further DataFrame transformation
         # this will catch any type error
         early_report = _validate(
@@ -457,7 +474,9 @@ class DataFrameSchematizer:
             k: schema["properties"][k]["items"].get("default")
             for k, v in schema["properties"].items()
         }
-        padded_columns = [colname for colname, value in _fillnas.items() if value == "_pad_"]
+        padded_columns = [
+            colname for colname, value in _fillnas.items() if value == "_pad_"
+        ]
         fillnas = {
             k: v for k, v in _fillnas.items() if k in df.columns and v is not None
         }
@@ -466,7 +485,7 @@ class DataFrameSchematizer:
         if padded_columns:
             df = df.replace({"_pad_": np.nan})
             df[padded_columns] = df[padded_columns].fillna(method="pad")
-            # df = df.fillna(method="pad", 
+            # df = df.fillna(method="pad",
         # =====================================================================
         # build dummy columns if required by the multiple-columns uniqueness
         # =====================================================================
@@ -648,7 +667,7 @@ class ValidatorMixin:
             self._schemas[tabname].add_columns(schema)
 
     def read_schema(self, *filepath):
-        """ assign a global schema by parsing the given filepath"""
+        """assign a global schema by parsing the given filepath"""
         if len(filepath) > 1:
             return self.read_multiple_yamls_schemas(*filepath)
         else:
@@ -674,37 +693,122 @@ class ValidatorMixin:
     def _validate_tab(self, tabname):
         """validate a tab using the provided scheme"""
         if tabname not in self._schemas:
-            return None, True, {}
-        return self._schemas[tabname].validate_dataframe(self._data[tabname])
+            return (None, True, {}), {}
+        schema = self._schemas[tabname]
+        result = schema.validate_dataframe(self._data[tabname])
+        return result, schema.xcalling
 
-    def validate(self):
+    def validate(self, row_reporting_offset="data"):
         """
         iterate through all tabs and validate eachone
+
+        row_reporting_offset can be used to offset rows reporting
         """
         # keep initial data before processing them
         if not hasattr(self, "_raw_data"):
             self._raw_data = {tabname: df.copy() for tabname, df in self._data.items()}
-        ret = {}
+        ret = defaultdict(dict)
+        xcallings = {}
         for tabname, df in self._data.copy().items():
-            df, is_ok, report = self._validate_tab(tabname)
+            (df, is_ok, report), xcalling = self._validate_tab(tabname)
+            if tabname in self._schemas:
+                if xcalling := self._schemas[tabname].xcalling:
+                    xcallings[tabname] = xcalling
             if df is None:
                 # tabname not described in schema; drop it
-                logging.info('drop tab "%s" which is not described in schema.', tabname)
+                logging.warning(
+                    'drop tab "%s" which is not described in schema.', tabname
+                )
                 self._data.pop(tabname)
                 continue
             self._data[tabname] = df  # override with filled (fillna) dataframe
             if not is_ok:
                 ret[tabname] = report
-        return ret
+        # ---------------------------------------------------------------------
+        # check cross-references calls
+        for src_tabname, targets in xcallings.items():
+            for src_colname, (target_tabname, target_colname) in targets.items():
+                schema = self._schemas[src_tabname].columns_specs[src_colname]
+                try:
+                    src = self._data[src_tabname][src_colname]
+                except KeyError:
+                    # colname is probably missing. This may not be an issue
+                    # if the given column is not mandatory
+                    if schema.get("mandatory"):
+                        logging.warning(
+                            f"cannot check xcallings for {src_tabname}/{src_colname}."
+                        )
+                    continue
+                else:
+                    target = self._data[target_tabname][target_colname]
+                    missing = src[
+                        ~src.isin(target)
+                    ].dropna()  # dropna since we may allow empty value
+                report = ret[src_tabname]
+                for row_nb, _missing_ref in missing.items():
+                    if (src_colname, row_nb) not in ret[src_tabname]:
+                        ret[src_tabname][(src_colname, row_nb)] = []
+                    ret[src_tabname][(src_colname, row_nb)].append(
+                        f"'value `{_missing_ref}` not found in `{target_tabname}::{target_colname}`"
+                    )
+        # ---------------------------------------------------------------------
+        # apply offset
+        if row_reporting_offset:
+            offset_ret = defaultdict(dict)
+            for tabname, report in ret.items():
+                is_units = len(self.getunits(tabname)) > 0
+                offset = _get_rows_offset(row_reporting_offset, is_units)
+                for index, _ in report.items():
+                    try:
+                        (colname, rownb) = index
+                        if rownb in (None, "?"):
+                            raise ValueError("switch...")
+                    except:
+                        offset_ret[tabname][index] = report[
+                            index
+                        ]  # "general" section, without row clue
+                    else:
+                        offset_ret[tabname][(colname, rownb + offset)] = report[
+                            (colname, rownb)
+                        ]
+
+            ret = offset_ret
+        return dict(ret)
 
     def dump_template(self):
         """return list of columns ready to be dumped as XLSX template"""
-        dic = {}
+        dic = defaultdict(dict)
         for tabname, schema in self._schemas.items():
-            dic[tabname] = pd.DataFrame({k: [] for k in schema.columns_specs.keys()})
+            df = pd.DataFrame(
+                {k: [v.get("units")] for k, v in schema.columns_specs.items()}
+            )
+            df.dropna(axis=0, how="all", inplace=True)
+            dic[tabname]["df"] = df
+            dic[tabname]["schema"] = schema.columns_specs
+            dic[tabname]["required"] = schema.required
+            dic[tabname]["xcalling"] = schema.xcalling
+            dic[tabname]["uniqueness"] = schema.uniqueness_sets
+        dic = dict(dic)
         return dic
 
 
+def _get_rows_offset(row_reporting_offset, is_units):
+    """by default (row_reporting_offset = 0),
+    row#0 means row#:
+                   | Data 0-based |Data 1-based | Spreadsheet  |
+    with units     | 0            |1            | 3            |
+    witthout units | 0            |1            | 2            |
+
+    """
+    if isinstance(row_reporting_offset, int):
+        offset = row_reporting_offset
+    elif row_reporting_offset == "data":
+        offset = 1  # equivalent to row_reporting_offset = 1
+    elif row_reporting_offset == "spreadsheet":
+        offset = 3 if is_units else 2
+    return offset
+
+
 if __name__ == "__main__":
     import doctest
 
