@@ -149,18 +149,17 @@ dh:
     }
     assert errors == expected_errors
     # check that values have been correctly filled
-    expected_df = pd.DataFrame(
-        {
-            "da": {1: None},
-            "db": {1: np.nan},
-            "dc": {1: 1},
-            "dd": {1: 1},
-            "de": {1: 2},
-            "df": {1: 2},
-            "dg": {1: 2},
-            "dh": {1: 2},
-        }
-    )
+    expected_data = {
+        "da": {0: None},
+        "db": {0: np.nan},
+        "dc": {0: 1},
+        "dd": {0: 1},
+        "de": {0: 2},
+        "df": {0: 2},
+        "dg": {0: 2},
+        "dh": {0: 2},
+    }
+    expected_df = pd.DataFrame(expected_data)
     assert pd.testing.assert_frame_equal(df, expected_df, check_dtype=False) is None
 
 
@@ -230,12 +229,12 @@ CF:
     assert errors == {}
     exp = pd.DataFrame(
         {
-            "CA": {1: 0.05, 2: 0.15},
-            "CB": {1: 5.3, 2: 1.0},
-            "CC": {1: 5.3, 2: 1.0},
-            "CD": {1: 241.53843370864516, 2: 0.0},
-            "CE": {1: "fwd", 2: "aft"},
-            "CF": {1: 0.9071847400000002, 2: 0.0},
+            "CA": {0: 0.05, 1: 0.15},
+            "CB": {0: 5.3, 1: 1.0},
+            "CC": {0: 5.3, 1: 1.0},
+            "CD": {0: 241.53843370864516, 1: 0.0},
+            "CE": {0: "fwd", 1: "aft"},
+            "CF": {0: 0.9071847400000002, 1: 0.0},
         }
     )
     assert pd.testing.assert_frame_equal(df, exp) is None
@@ -291,9 +290,10 @@ def test_VData_00():
         ("life_nb", 1): ["'hg' is not of type 'integer'"],
         ("life_nb", 2): ["15 is greater than the maximum of 4"],
     }
-    df, is_valid, errors = data._validate_tab("test")
+    (df, is_valid, errors), xcalling = data._validate_tab("test")
     assert is_valid is False
     assert errors == expected_report
+    assert xcalling == {}
 
 
 # =============================================================================
@@ -323,15 +323,10 @@ def test_IVData_00(datadir):
     data.read_schema(os.path.join(indir, "test_00.schema.yaml"))
     for tab in ("names", "cars", "empty"):
         print('checking "%s"' % tab, end="... ")
-        df, is_ok, errors = data._validate_tab(tab)
-        try:
-            assert is_ok is True
-        except:
-            print("OUPS!")
-            pp(errors)
-        else:
-            print("OK")
+        (df, is_ok, errors), xcalling = data._validate_tab(tab)
+        assert is_ok is True
         assert errors == {}
+        assert xcalling == {}
     # ------------------------------------------------------------------------
     # export and reimport to/from various formats
     for extension in (".cfg", ".xlsx", ".ini"):
@@ -347,7 +342,7 @@ def test_IVData_00(datadir):
         data_new.read_schema(os.path.join(indir, "test_00.schema.yaml"))
         for tab in data._data.keys():
             print('checking "%s"' % tab, end="... ")
-            df, is_ok, errors = data_new._validate_tab(tab)
+            (df, is_ok, errors), xcalling = data_new._validate_tab(tab)
             try:
                 assert is_ok is True
             except:
@@ -371,9 +366,20 @@ def test_IVData_02(datadir):
     data = IVData()
     data.read_excel(os.path.join(indir, "test_00.xlsx"))
     data.read_schema(os.path.join(indir, "test_00.schema2.yaml"))
+    # default row reporting as "data"
     ret = data.validate()
     assert ret == {
-        "cars": {("Year", 2): ["None is not of type 'integer'"]},
+        "cars": {("Year", 3): ["None is not of type 'integer'"]},
+        "names": {"general": ["'name' is a required property"]},
+    }
+    ret = data.validate(10)
+    assert ret == {
+        "cars": {("Year", 12): ["None is not of type 'integer'"]},
+        "names": {"general": ["'name' is a required property"]},
+    }
+    ret = data.validate("spreadsheet")
+    assert ret == {
+        "cars": {("Year", 4): ["None is not of type 'integer'"]},
         "names": {"general": ["'name' is a required property"]},
     }
 
@@ -384,13 +390,13 @@ def test_IVData_03(datadir):
     data = IVData()
     data.read_excel(os.path.join(indir, "test_02.xlsx"))
     data.read_schema(os.path.join(indir, "test_02.schema.yaml"))
-    ret = data.validate()
+    ret = data.validate("data")
     print(ret)  # TODO: remove me!
     assert len(ret) == 2
     expected_errors = {
         "names": {"general": ["'life_nb' is a required property"]},
         "french_cars": {
-            ("brand", 2): [
+            ("brand", 3): [
                 "'Citroën' is not one of ['Peugeot', 'Toyota', 'Ford', 'Renault']"
             ]
         },
@@ -440,16 +446,17 @@ def test_IVData_04(datadir):
         "type": "object",
     }
     assert data._schemas["geom"].build() == expected_schema
-    ret = data.validate()
+    ret = data.validate("spreadsheet")
     expected_geom = pd.DataFrame(
         {
-            "id": {1: 1.0, 2: 2.0},
-            "distA": {1: 0.0010000000000000005, 2: 0.002000000000000001},
-            "distB": {1: 14.3, 2: 15.0},
-            "Volume": {1: 1.5064562386943998, 2: 0.8183568665087998},
+            "id": {0: 1.0, 1: 2.0},
+            "distA": {0: 0.0010000000000000005, 1: 0.002000000000000001},
+            "distB": {0: 14.3, 1: 15.0},
+            "Volume": {0: 1.5064562386943998, 1: 0.8183568665087998},
         }
     )
-    assert pd.testing.assert_frame_equal(data._data["geom"], expected_geom) is None
+    actual_geom = data._data["geom"]
+    assert pd.testing.assert_frame_equal(actual_geom, expected_geom) is None
 
 
 def test_IVData_05_failing_units(datadir):
@@ -552,10 +559,10 @@ def test_IVData_05_failing_units(datadir):
     }
 
     assert data._schemas["volume"].build() == expected_schema
-    ret = data.validate()
+    ret = data.validate("data")
     expected_report = {
         "mass": {
-            ("test_mass_g", 2): ["None is not of type 'number'"],
+            ("test_mass_g", 3): ["None is not of type 'number'"],
             ("test_mass_lbm", None): ["undefined units 'lbm'"],
         },
         "volume": {"general": ["'test_volume_foot3' is a required property"]},
@@ -696,7 +703,7 @@ def test_IVData_07_failing_units(datadir):
                 "'test_mass_tonne' is a required property",
                 "'test_mass_g' is a required property",
             ],
-            ("test_mass_lb", 0): [
+            ("test_mass_lb", 1): [
                 "'a' is not valid under any of the given " "schemas",
                 "'a' is not of type 'number'",
                 "'a' is not of type 'null'",
@@ -734,7 +741,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
     df = data_si.get("WithPadding_ok")
     expected_df = pd.DataFrame(
         {
-            1: {
+            0: {
                 "another": 1.0,
                 "block_label": "RECARO1",
                 "block_ref_point_id": 2.0,
@@ -742,7 +749,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
                 "rail_tag": "RBL11",
                 "x_loc": 3500.0,
             },
-            2: {
+            1: {
                 "another": 2.0,
                 "block_label": "RECARO1",
                 "block_ref_point_id": 2.0,
@@ -750,7 +757,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
                 "rail_tag": "RBL11",
                 "x_loc": 8500.0,
             },
-            3: {
+            2: {
                 "another": 3.0,
                 "block_label": "s1",
                 "block_ref_point_id": 2.0,
@@ -758,7 +765,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
                 "rail_tag": "RBL12",
                 "x_loc": 3800.0,
             },
-            4: {
+            3: {
                 "another": 4.0,
                 "block_label": "s2",
                 "block_ref_point_id": 2.0,
@@ -766,7 +773,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
                 "rail_tag": "RBL12",
                 "x_loc": 3800.0,
             },
-            5: {
+            4: {
                 "another": 5.0,
                 "block_label": "s1",
                 "block_ref_point_id": 2.0,
@@ -774,7 +781,7 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
                 "rail_tag": "RBL12",
                 "x_loc": 8500.0,
             },
-            6: {
+            5: {
                 "another": 6.0,
                 "block_label": "s1",
                 "block_ref_point_id": 2.0,
@@ -786,3 +793,27 @@ def test_IVData_08_multi_cols_uniqueness(datadir):
     ).T
     expected_df = expected_df[[c for c in df]]
     assert pd.testing.assert_frame_equal(df, expected_df, check_dtype=False) is None
+
+
+def test_IVData_09_must_exist_in(datadir):
+    """check must exist req"""
+    indir, outdir = datadir
+    data = IVData()
+    data.read_excel(os.path.join(indir, "test_06_must_exist_in.xlsx"))
+    data.read_schema(os.path.join(indir, "test_06_must_exist_in.yaml"))
+    ret = data.validate("spreadsheet")
+    expected_errors = {
+        "beams": {
+            ("beam_id", 5): ["'value `6.0` not found in `beam_def::id`"],
+            ("color_id", 7): [
+                "'a' is not of type 'integer'",
+                "'value `a` not found in `colors::id`",
+            ],
+        },
+        "colors": {
+            ("id", 4): ["'tata' is not of type 'integer'"],
+            ("name", 4): ["5 is not of type 'string'"],
+        },
+    }
+
+    assert ret == expected_errors
